signature L0Theory =
sig
  type thm = Thm.thm
  
  (*  Definitions  *)
    val L00_def : thm
    val REF : thm
    val REF_TY_DEF : thm
    val REF_case_def : thm
    val REF_repfns : thm
    val REF_size_def : thm
    val none : thm
    val void : thm
    val void_BIJ : thm
    val void_TY_DEF : thm
    val void_case_def : thm
    val void_size_def : thm
  
  (*  Theorems  *)
    val REF_11 : thm
    val REF_Axiom : thm
    val REF_case_cong : thm
    val REF_induction : thm
    val REF_nchotomy : thm
    val num2void_11 : thm
    val num2void_ONTO : thm
    val num2void_void2num : thm
    val void2num_11 : thm
    val void2num_ONTO : thm
    val void2num_none : thm
    val void2num_num2void : thm
    val void2num_void : thm
    val void_Axiom : thm
    val void_EQ_void : thm
    val void_case_cong : thm
    val void_distinct : thm
    val void_induction : thm
    val void_nchotomy : thm
  
  val L0_grammars : type_grammar.grammar * term_grammar.grammar
(*
   [integer] Parent theory of "L0"
   
   [string] Parent theory of "L0"
   
   [L00_def]  Definition
      
      |- L00 = (\a. mk_REF ((\a. CONSTR 0 a (\n. BOTTOM)) a))
   
   [REF]  Definition
      
      |- REF = L00
   
   [REF_TY_DEF]  Definition
      
      |- ?rep.
           TYPE_DEFINITION
             (\a0.
                !'REF'.
                  (!a0.
                     (?a. a0 = (\a. CONSTR 0 a (\n. BOTTOM)) a) ==>
                     'REF' a0) ==>
                  'REF' a0) rep
   
   [REF_case_def]  Definition
      
      |- !f a. case f (REF a) = f a
   
   [REF_repfns]  Definition
      
      |- (!a. mk_REF (dest_REF a) = a) /\
         !r.
           (\a0.
              !'REF'.
                (!a0.
                   (?a. a0 = (\a. CONSTR 0 a (\n. BOTTOM)) a) ==>
                   'REF' a0) ==>
                'REF' a0) r =
           (dest_REF (mk_REF r) = r)
   
   [REF_size_def]  Definition
      
      |- !f a. REF_size f (REF a) = 1 + f a
   
   [none]  Definition
      
      |- none = num2void 1
   
   [void]  Definition
      
      |- void = num2void 0
   
   [void_BIJ]  Definition
      
      |- (!a. num2void (void2num a) = a) /\
         !r. (\n. n < 2) r = (void2num (num2void r) = r)
   
   [void_TY_DEF]  Definition
      
      |- ?rep. TYPE_DEFINITION (\n. n < 2) rep
   
   [void_case_def]  Definition
      
      |- (!x0 x1. (case void of void -> x0 || none -> x1) = x0) /\
         !x0 x1. (case none of void -> x0 || none -> x1) = x1
   
   [void_size_def]  Definition
      
      |- !x. void_size x = 0
   
   [REF_11]  Theorem
      
      |- !a a'. (REF a = REF a') = (a = a')
   
   [REF_Axiom]  Theorem
      
      |- !f. ?fn. !a. fn (REF a) = f a
   
   [REF_case_cong]  Theorem
      
      |- !M M' f.
           (M = M') /\ (!a. (M' = REF a) ==> (f a = f' a)) ==>
           (case f M = case f' M')
   
   [REF_induction]  Theorem
      
      |- !P. (!a. P (REF a)) ==> !R. P R
   
   [REF_nchotomy]  Theorem
      
      |- !R. ?a. R = REF a
   
   [num2void_11]  Theorem
      
      |- !r r'.
           r < 2 ==> r' < 2 ==> ((num2void r = num2void r') = (r = r'))
   
   [num2void_ONTO]  Theorem
      
      |- !a. ?r. (a = num2void r) /\ r < 2
   
   [num2void_void2num]  Theorem
      
      |- !a. num2void (void2num a) = a
   
   [void2num_11]  Theorem
      
      |- !a a'. (void2num a = void2num a') = (a = a')
   
   [void2num_ONTO]  Theorem
      
      |- !r. r < 2 = ?a. r = void2num a
   
   [void2num_none]  Theorem
      
      |- void2num none = 1
   
   [void2num_num2void]  Theorem
      
      |- !r. r < 2 = (void2num (num2void r) = r)
   
   [void2num_void]  Theorem
      
      |- void2num void = 0
   
   [void_Axiom]  Theorem
      
      |- !x0 x1. ?f. (f void = x0) /\ (f none = x1)
   
   [void_EQ_void]  Theorem
      
      |- !a a'. (a = a') = (void2num a = void2num a')
   
   [void_case_cong]  Theorem
      
      |- !M M' x0 x1.
           (M = M') /\ ((M' = void) ==> (x0 = x0')) /\
           ((M' = none) ==> (x1 = x1')) ==>
           ((case M of void -> x0 || none -> x1) =
            case M' of void -> x0' || none -> x1')
   
   [void_distinct]  Theorem
      
      |- ~(void = none)
   
   [void_induction]  Theorem
      
      |- !P. P none /\ P void ==> !a. P a
   
   [void_nchotomy]  Theorem
      
      |- !a. (a = void) \/ (a = none)
   
   
*)
end
