(* =============================================================

   The specification of an ML-test-engine for L0min

============================================================= *)

local

open Parse HolKernel stringLib stringSyntax
open PP util_PP
open L0_emb_mldata L0_emb_basic
open L0min_2ml_Translator Test_Engine Test_Engine_gen
(* open L0min_Syntax *)

in

structure L0min_mlte : Test_Engine =

   struct

   local
 
   structure L0Min2ML = L0min2ml_spec

   fun ppInstr i pps = fold_INSTR L0Min2ML.tAlgebra (buildINSTRtree i) pps

   fun pp_decl_rndvar v =
       let
       val ty = (fst o dest_type o type_of) v
       fun pprhs pps = if ty="int"         then add_string pps "ref (int_rnd rgen__)"
                       else if ty="bool"   then add_string pps "ref (bool_rnd rgen__)"
                       else if ty="string" then add_string pps "ref (string_rnd rgen__)"
                       else (* then ty=void *) add_string pps "ref ()"
       in
       ( (fn pps => (add_string pps o fst o dest_var) v)
       , pprhs )
       end

   fun ppInstr_with_randomize_freevars i pps
       =
       pp_ml_let (List.map pp_decl_rndvar (free_vars i)) (ppInstr i) pps


   in 

   type TEoption = unit
   val  option   = ref ()

   val teAlg = { ftest = let
                         fun pptest _ name i path pps =
                               ( L0Min2ML.option := { topt_skipassert = false } 
                                 (* switching option so that "assert" is not skipped *)

                               ; pp_ml_handle (ppInstr_with_randomize_freevars i)
         
                                       [ ((fn pps => add_string pps "Fail s")
                                         ,(fn pps => ( begin_block pps INCONSISTENT 0
                                                     ; add_string pps ("( print \"\\nTest " ^ path ^ "." ^ name ^ " fail:\"")
                                                     ; add_newline pps
                                                     ; add_string pps "; print (\"\\n   \" ^ s))" 
                                                     ; end_block pps 
                                                     )
                                           ))
                                       ] 

                                       pps
                               ) 
                         in
                         pptest end

               , fsuite = let
                          fun ppsuite _ name rs path pps =
                               ( pp_list (fn pps => add_string pps "; ") 
                                         (fn pps => add_string pps "(")
                                         (fn pps => add_string pps ")")
                                         (fn pps => add_string pps "()") 
                                         (List.map (fn r=> r (path ^ "." ^ name)) rs)
                                          pps
                               )
                         in ppsuite end
               }

   fun ppte ppsuite pps = ( begin_block pps CONSISTENT 0
                          ; add_string pps "load \"Random\";" ; add_newline pps
                          ; add_string pps "load \"Test_randomgen\";" ; add_newline pps
                          ; add_string pps "open Random Test_randomgen" 
                          ; add_newline pps ; add_newline pps 
                          ; add_string pps "fun test rgen__ = "
                          ; add_newline pps 
                               ; indent 4 pps
                               ; ppsuite pps
                          ; add_newline pps ; add_newline pps 
                          ; add_string pps "fun runtest_ n rgen = "             ; add_newline pps 
                          ; add_string pps "    if n <= 0 then ()"              ; add_newline pps 
                          ; add_string pps "    else (test rgen; runtest_ (n-1) rgen)" 
                          ; add_newline pps ; add_newline pps 
                          ; add_string pps "fun runtest n rseed = runtest_ n (newgenseed rseed)"
                          ; end_block pps )

   end (* of inner local *)
   end (* of struct *)


(* ===================================================================

  Now generating the coresponding Test-engine

=================================================================== *)

structure L0min_MLTE = build_TE(L0min_mlte) ;
 
end 

