(* ===================================================================

  Some common types and functions to build logics 

  author: wishnu prasetya

=================================================================== *)

local 

open Parse HolKernel boolLib 

open Option Err L0Theory L0_emb_mldata L0_emb_basic ;

fun matchTerm t1 t2 = SOME (fst (match_term t1 t2)) 
                      handle _ => NONE ;

fun snd (_,y) = y ;
fun fst (x,_) = x ;

in

(* ===================================================================

  Some useful, often used, small functions

=================================================================== *)


infixr 5 AND OR ;
infixr 4 IMP ;


fun p AND q = mk_conj (p,q) ;
fun p OR  q = mk_disj (p,q) ;
fun p IMP q = mk_imp  (p,q) ;
fun NOT p   = mk_neg p ;
val mk_T    = Term `T` ;
val mk_F    = Term `F` ;


(* ===================================================================

   substitutions, fresh variables

=================================================================== *)

(* --- substx (xs,ys) e = e[xs/ys] *)

fun substx (xs,ys) e
    = 
    let
    val the_subst = List.map (fn (x,y)=> {redex = x , residue = y })
                             (zip ys xs)
    in
    subst the_subst e
    end ;

fun mkFreshVars xs = List.map (genvar o type_of) xs ;

(* --- replace xs with fresh vars *)

fun substxFresh xs e
    =
    substx (mkFreshVars xs, xs) e ;

(* --- operators for substitution, such that:

       e <- (xs//ys) <- (as//bs)    =   e[xs/ys][as/bs]

*)

infixr   //  ;
infix    <-  ;

fun x // y = (x,y)
fun e <- s = substx s e


fun mk_ret retty = mk_const ("ret",retty) 

(* --- constructing an "old .. " expression *)

fun mk_old t = 
    let
    val ty = type_of t
    in
    mk_comb (mk_const ("old", ty --> ty), t)
    end 

fun mk_olds ts = List.map mk_old ts


(* --- recognizing and constructing an "REF .. " expression *)

fun is_ref t = (fst o dest_const o rator) t  = "REF"
                handle
                _  =>  false

fun mk_ref t = 
    let
    val ty = type_of t
    in
    mk_comb (mk_const ("REF", ty --> Type `:REF`), t)
    end 



(* ===================================================================

   function to instantiate the definition of a program

=================================================================== *)

fun get_formal_rparams t = 
    case matchTerm (Term `jhgdsjg hjgasj jkjakjh = kjhjhh`) t of
         SOME match 
              =>
              (dest_params o #residue o el 2) match
    ;

fun get_formal_vparams t = 
    case matchTerm (Term `jhgdsjg hjgasj jkjakjh = kjhjhh`) t of
         SOME match 
              =>
              (dest_params o #residue o el 3) match
    ;

fun TY_INST_prog_def     
         aparams      (* list of actual params *)
         prog_def     (* theorem stating the source progam definition;
                         it should be in the form |- foo (...) = ...
                      *)

         (* this program will return the same definition, but
            type-instantiated to match the actual params        *)
         =
         let
         fun isREF t  = isSome (matchTerm (Term `REF jhgdsjg`) t) 

         fun stripREF t = if isREF t then rand t else t

         val aparams_   = List.map stripREF aparams

         val (_,fparams,_,_,_) = xdestProgDef (concl prog_def)

         val fparams_   = List.map stripREF fparams

         val th1 = List.foldr (fn (v,th) => GEN v th) prog_def fparams_

         val fparams2 = List.map (fn (f,a) 
                                  => 
                                  mk_var (fst (dest_var f), type_of a))
                                 (zip fparams_ aparams_)

         val th2 = ISPECL fparams2 th1
         in
         th2
         end 
         ;


(* verification condition *)

type VC  = { VClabel : string, VC : term } ;

(* ---
   an instance of guessInv (see logic.sig) that refuses to guess invariant; 
   can be used to initialize guessInv. 
--- *)

val (noguessInv : term -> term -> term)
    =
    let
    fun f loop postC 
        =
        raise (L0Exn { errorCode   = SOME "noguessInv"
                     , errorWhere  = NONE
                     , errorInside = SOME loop
                     , errorWhy    = SOME "an invariant sould be specified" } )
    in
    f
    end ;

(* ---
   a simple instance of getSpec (see logic.sig); can be used to intialize getSpec

--- *)

fun simple_getSpec prg_name
    =
    theorem (prg_name ^ "_def") 
    handle
    _ => definition (prg_name ^ "_def") 


end ;


(* ===================================================================

POSTPONED:

   Introduce a protected data-type to represent a certified specification. 
   The second element is a theorem this is the 'certificate' of the 
   specification's validity. Later, the only way to produce CERT is 
   via the WP-reduction given later.

datatype CERT = CERT of (term * thm) ;

fun getSpec (CERT (s,_)) = s ;
fun getCert (CERT (_,c)) = c ;

=================================================================== *)
