local 

open PP

fun replicate n x = if n<=0 then [] else x :: replicate (n-1) x

in

fun ppempty pps  = ()

fun indent n pps =  (add_string pps o String.implode o replicate n) (#" ")

(* --- for pretty printing binary and unary operators *)

fun pp_binop binopName ppx ppy pps = ( begin_block pps INCONSISTENT 0
                                 ; add_string pps "("
                                 ; ppx pps
                                 ; add_break pps (1,1)
                                 ; add_string pps binopName
                                 ; add_break pps (1,1)
                                 ; ppy pps
                                 ; add_string pps ")" 
                                 ; end_block pps )

fun pp_unop unopName ppx pps   = ( add_string pps "("
                                 ; add_string pps unopName
                                 ; add_string pps " "
                                 ; ppx pps
                                 ; add_string pps ")" )

(* --- for pretty printing a list-like structure *)

fun pp_list pp_sep pp_left pp_right pp_empty ts pps =
    let
    fun list2 pps [t]    = t pps
      | list2 pps (t::z) = ( t pps
                               ; add_break  pps (0,0)
                               ; pp_sep pps ; add_string pps " "
                               ; list2 pps z )
   in
   case ts of
       []   =>   pp_empty pps
     | z    =>   ( pp_left pps
                  ; begin_block pps CONSISTENT 0
                  ; list2 pps z
                  ; pp_right pps
                  ; end_block pps )
   end


(* --- ML if-then-else  *)

fun pp_ml_if ppguard ppthen ppelse pps 
   
     = ( add_string pps "if "
       ; begin_block pps CONSISTENT 0
       ; ppguard pps 
       ; add_break pps (1,0)
       ; add_string pps "then "
       ; ppthen pps
       ; add_break pps (1,0)
       ; add_string pps "else "
       ; ppelse pps 
       ; end_block pps )   

(* --- ML while  *)

fun pp_ml_while ppguard ppbody pps 
   
     = ( add_string pps "while "
       ; begin_block pps CONSISTENT 0
       ; ppguard pps 
       ; add_break pps (1,0)
       ; add_string pps "do"
       ; add_break pps (1,0)
       ; ppbody pps
       ; end_block pps )   

(* --- ML execption handle *)

fun pp_ml_handle ppexpr pppairs_handler pps =
     let
     fun pph (pppat,pphandler) pps = ( pppat pps 
                                     ; add_string pps " => "
                                     ; pphandler pps )
     in
     ( begin_block pps CONSISTENT 0
     ; ppexpr pps 
     ; add_break pps (1,0)
     ; add_string pps "handle"
     ; add_break pps (1,0)
     ; pp_list (fn pps => (add_newline pps; add_string pps "|"))
                ppempty
                ppempty
                ppempty
               (List.map pph pppairs_handler)
                pps
     ; end_block pps )
     end

(* --- ML let  *)

fun pp_ml_let pppairs_decl ppbody pps =
    let
    fun ppl (pplhs,pprhs) pps = ( add_string pps "val "
                            ; pplhs pps
                            ; add_string pps " = "
                            ; pprhs pps )
    in
    ( begin_block pps CONSISTENT 3
    ; add_string pps "let"
    ; add_newline pps
    ; pp_list add_newline 
              ppempty 
              ppempty
              ppempty
              (List.map ppl pppairs_decl)
              pps
    ; add_newline pps
    ; add_string pps "in"
    ; add_newline pps
    ; ppbody pps
    ; add_newline pps
    ; add_string pps "end"
    ; end_block pps )
    end

end