open intLib stringLib pred_setLib ;
open linguTheory ;
open L0Theory      
     L0min_Logic  
     L0min_test_verifier ;

local

open Parse HolKernel boolLib bossLib ;

val lingu_normalizer_defs = [map_def, only_def, drop_def, satisfy_def]
                            @
                            [ ALLof_ins_thm, SOMEof_ins_thm, IN_ins_thm
                            , ALLof_insert_thm, SOMEof_insert_thm, IN_insert_thm
                            , ALLof_delete_thm, SOMEof_delete_thm, IN_delete_thm
                            , ALLof_update_thm, SOMEof_update_thm, IN_update_thm
                            , ALLof_select_thm, SOMEof_select_thm, IN_select_thm
                            ] ;



in

(* ===================================================================

   A pre-processor to unfold user-defined constants/functions and
   to normalize Lingu-specific instructions to L0 syntax.

=================================================================== *)

fun lingu_preprocess_CONV user_definitions 
    =  
    REWRITE_CONV user_definitions
    THENC REWRITE_CONV [ins_def, insert_def, del_def, delete_def, update_def] ;

(* some short-hand for reduce-functions *)

fun reduce user_definitions linguProg_def 
    = 
    ( L0min_vcg.reduce 
    o CONV_RULE (lingu_preprocess_CONV user_definitions)) linguProg_def ;

fun test_reduce user_definitions test 
    = 
    ( L0min_sevcg.reduce (lingu_preprocess_CONV user_definitions)
    o rand
    o concl
    o lingu_preprocess_CONV user_definitions ) 
      test ;

(* ===================================================================

   Normalizer to reduce Lingu-specific expressions to standard HOL
   terms. They should be applied as post-processor to the generated
   VCs.

=================================================================== *)

val lingu_normalizer_CONV =
    REWRITE_CONV lingu_normalizer_defs
    THENC REWRITE_CONV [ ALLof_def, SOMEof_def
                       , deleteX_def, insertX_def, updateX_def, select_def
                       , IN_DEF ]
    THENC DEPTH_CONV BETA_CONV ;

val lingu_normalizer_TAC = CONV_TAC lingu_normalizer_CONV ;

(* ===================================================================

   Etc stuffs

=================================================================== *)


val set_ss = std_ss ++ pred_setLib.PRED_SET_ss ;


end ;


